<?php
/**
 * Copyright 2012-2016 Horde LLC (http://www.horde.org/)
 *
 * See the enclosed file LICENSE for license information (ASL).  If you
 * did not receive this file, see http://www.horde.org/licenses/apache.
 *
 * @author  Michael Bunk <mb@computer-leipzig.com>
 * @author  Jan Schneider <jan@horde.org>
 * @package Ingo
 */

/**
 * Ingo_Transport_Interworx implements an Ingo transport driver to allow
 * scripts to be installed and set active on an Interworx server.
 *
 * Ingo_Transport_
 *
 * @author  Michael Bunk <mb@computer-leipzig.com>
 * @author  Jan Schneider <jan@horde.org>
 * @package Ingo
 */

require_once '/home/interworx/lib/pear/DB.php';
require_once '/home/interworx/include/IWorxDebug.php';
require_once '/home/interworx/include/IWorxDbUtil.php';

class Ingo_Transport_Interworx extends Ingo_Transport_Base
{
    /**
     * The current vacation details.
     *
     * @var array
     */
    protected $_details = null;

    private $_dsn = array();

    /**
     * Constructor.
     *
     * @param array $params  Configuration parameters.
     */
    public function __construct(array $params = array())
    {
      parent::__construct( $params );
      $iworx_ini = parse_ini_file( '/home/interworx/lib/horde/iworx.ini', true );
      $this->_dsn = $iworx_ini['vpopmail']['dsn'];
    }

    /**
     * Sets a script running on the backend.
     *
     * @param array $script  The filter script information. Passed elements:
     *                       - 'name': (string) the script name.
     *                       - 'recipes': (array) the filter recipe objects.
     *                       - 'script': (string) the filter script.
     *
     * @throws Ingo_Exception
     */
    public function setScriptActive($script)
    {
        $vacation = null;
        foreach ($script['recipes'] as $recipe) {
            if ($recipe['rule'] == Ingo::RULE_VACATION) {
                $vacation = $recipe['object']->vacation;
                break;
            } else {
                throw new Ingo_Exception('The Interworx transport driver only supports vacation rules.');
            }
        }

        if (!$vacation) {
            return;
        }
        $this->_getUserDetails($this->_params['password']);
        $this->_details['message'] = Ingo_Script_Util::vacationReason(
            $vacation->getVacationReason(),
            $vacation->getVacationStart(),
            $vacation->getVacationEnd()
        );

        $this->_details['message'] = $vacation->getVacationReason();

        if( $recipe['object']->disable ) {
            $this->_unsetVacation();
        } else {
            $this->_setVacation();
        }
    }

    /**
     * Setup vacation notices for a user.
     *
     * @return boolean  Returns true on success, false on error.
     */
    protected function _setVacation()
    {
        $user   = $this->_details['username'];
        $domain = $this->_details['domain'];
        $message = $this->_details['message'];

        $domain = $this->_translatePointerDomain( $domain );

        $tmpfname = tempnam("", "VACA");
        $handle = fopen($tmpfname, "w");
        fwrite($handle, $message);
        fclose($handle);

        $cmd = "/home/interworx/lib/horde/setvacation --set-vacation-message=$tmpfname --user=$user --domain=$domain 2>&1";

        exec( $cmd, $result, $retval );

        $db = IWorxDbUtil::db_connect( $this->_dsn );

        $sql = "DELETE FROM valias
                WHERE alias = '$user' AND
                domain = '$domain' AND
                valias_line LIKE '|%vacation.sh%'";
        $db->query( $sql );

        $sql = "INSERT INTO valias (alias, domain, valias_line)
                VALUES ('$user', '$domain', '|/home/interworx/lib/autorespond/vacation.sh' )";
        $db->query( $sql );

        $sql = "SELECT count(*) from valias
                WHERE alias = '$user' AND
                domain = '$domain' AND
                valias_line LIKE '|%deliverquota.sh%'";
        $exists = $db->getOne( $sql );

        if( $exists == 0 ) {
            $sql = "INSERT INTO valias (alias, domain, valias_line)
                    VALUES ('$user', '$domain', '|/home/interworx/lib/autorespond/deliverquota.sh' )";
            $db->query( $sql );
        }

        unlink( $tmpfname );
        return true;
    }

    /**
     * Disables vacation notices for a user.
     *
     * @param string    $user       The user to disable vacation notices for.
     * @param string    $realm      The realm of the user.
     * @param string    $pass       The password of the user.
     *
     * @return boolean  Returns true on success, false on error.
     */
    protected function _unsetVacation()
    {
        $domain = $this->_details['domain'];
        $user   = $this->_details['username'];

//      if( $realm == 'default' ) {
//          $regs = array();
//          ereg( "@(.*)", Auth::getAuth(), $regs );
//          $domain = $regs[1];
//      }

        $domain = $this->_translatePointerDomain( $domain );

        $db = IWorxDbUtil::db_connect( $this->_dsn );
        $sql = "DELETE FROM valias
                WHERE alias = '$user' AND
                domain = '$domain' AND
                valias_line LIKE '|%vacation.sh%'";
        $db->query( $sql );

        $sql = "SELECT count(*)
                FROM valias
                WHERE alias = '$user' AND
                domain = '$domain' AND
                valias_line like '&%'";

        $exists = $db->getOne( $sql );
        if( $exists == 0 ) {
            $sql = "DELETE FROM valias
                    WHERE alias = '$user' AND
                    domain = '$domain' AND
                    valias_line LIKE '|%deliverquota.sh%'";
            $db->query( $sql );
        }
        return true;
    }


    /**
     * Retrieves the current vacation details for the user.
     *
     * @param string $password  The password for user.
     *
     * @return array  Vacation details
     * @throws Ingo_Exception
     */
    protected function _getUserDetails($password)
    {
        global $registry;
        if (!is_null($this->_details)) {
            return $this->_details;
        }

	$username = $this->_params['username'];

    preg_match('/@(.*)/', $registry->getAuth(), $regs);
	$domain = $regs[1];

        $this->_details['username'] = $this->_params['username'];
	$this->_details['password'] = $password;
	$this->_details['domain']   = $domain;
	$this->_details['vacation'] = $this->_isEnabled( $username, $password ) ? 'Y' : 'N';
	$this->_details['message']  = $this->_currentMessage( $username, $password );

        return $this->_details;
    }

    /**
     * Retrieves current state of vacation notices for a user.
     *
     * @param string    $user       The user to check vacation status of.
     * @param string    $realm      The realm of the user.
     * @param string    $pass       The password of the user.
     *
     * @return mixed    Returns true if vacation  is enabled for the user
     *                  or false if vacation is currently disabled.
     */
    protected function _isEnabled($user, $password)
    {
        $domain = preg_split( '/@/', $user )[1];
        $domain = $this->_translatePointerDomain( $domain );

        $db = IWorxDbUtil::db_connect( $this->_dsn );
        $sql = "SELECT COUNT(*) FROM valias WHERE alias = '$user' AND domain = '$domain' AND valias_line LIKE '|%vacation.sh%'";
        $cnt = $db->getOne( $sql );
        if( $cnt > 0 ) {
            return( 'Y' );
        } else {
            return( 'N' );
        }
    }

    /**
     * Get the parent domain if this is a master domain.
     *
     * @param domain
     */
    protected function _translatePointerDomain( $domain )
    {
        if( is_readable( '/var/qmail/users/assign' ) ) {
            // check for pointer domains
            $assign = file( '/var/qmail/users/assign' );
            foreach( $assign as $line ) {
                $parts = preg_split( '/:/', $line );
                if( $parts[0] === "+{$domain}-" ) {
                    $domain = $parts[1];
                }
            }
        }
        return $domain;
    }

    /**
     * Retrieves current vacation setup.
     *
     * @param string    $user       The user to fetch the setup from.
     * @param string    $password   The password of the user.
     *
     * @return string   Returns a string containing the current forwarding mail address or
     *                  false on error.
     */
    protected function _currentMessage($user, $password)
    {
      global $registry;

      $domain = $this->_details['domain'];
      $domain = $this->_translatePointerDomain( $domain );

      assert( $domain != "" );
      $cmd = "/home/vpopmail/bin/vuserinfo -d {$user}@{$domain}";
      exec( $cmd, $result, $retval );
      $userdir = $result[0];
      if( @is_file( "{$userdir}/vacation" ) ) {
	return( $this->_stripHeaders(@file_get_contents( "{$userdir}/vacation" )) );
      } else {
	return( "I'm on vacation and will not be reading my mail for a while.\nYour mail will be dealt with when I return." );
      }
      return false;
    }

    private function _stripHeaders($message)
    {
        $parts = preg_split('~\R\R~', $message, 2);
        if(count($parts) === 2) {
            return $parts[1];
        } else {
            return '';
        }
    }

}
